/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag
 * 
 * Code for Algorithm 2.11 (Box-Muller), p. 38 
 *
 * this file: algo02_11.c
 *
 * For a demo how to use this algorithm see the file "demo_algo02_11.c".
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "uniform.h"

#ifndef M_PI
#define M_PI       3.14159265358979323846264338328      /* pi */
#endif

double generator02_11(void)
/*
 * Algorithm 2.11 (Box-Muller), p. 38  
 */
{ double u, v, r, y;
  static double x;
/*1*/ static int stored=0;
/*2*/ if(stored){
/*3*/   stored = 0;
/*4*/   return x;}
/*5*/ else{
/*6*/   u = uniform();
        v = uniform();
/*7*/   r = sqrt(-2.*log(v));
/*8*/   x = r*cos(2.*M_PI*u);
/*9*/   y = r*sin(2.*M_PI*u);
/*10*/  stored = 1;
/*11*/  return y;
      }
}




