/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Code for Algorithms 3.3 & 3.4 (Alias sample and setup), p. 48
 *
 * this file: algo03_03.c
 *
 * For a demo how to use this algorithm see the file "demo_algo03_03.c".
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "uniform.h"

  static double *q=NULL;
  static int *a=NULL,LL;

int setup03_03(double (*pmf)(int), int L)
/*
 * Algorithm 3.4 (Alias setup for Algorithm 3.3), p. 48
 *
 * pmf... function pointer to probability (mass) function for k >= 0
 * L ... non zero probabilities assumed for (0, 1, 2, ..., L-1)
 *
 */
{ 
  int l,k,greateri=-1,smalleri=-1;
  int *greater, *smaller;


  
      if(q!=NULL){
	free(a);
	free(q);
      }
      LL = L;
      a = malloc(L*sizeof(int));
      q = malloc(L*sizeof(double));
      greater = malloc(L*sizeof(int));
      smaller = malloc(L*sizeof(int));
/*1*/ for(l=0;l<L;l++){
/*2*/   q[l] = L*(*pmf)(l);
/*3*/   if(q[l]>=1.){
	  greateri++;
	  greater[greateri] = l;
	}
	else{
	  smalleri++;
	  smaller[smalleri] = l;
	}
      }
/*4*/ while(smalleri >= 0 && greateri >= 0){
/*5*/   k = greater[greateri];
        l = smaller[smalleri];
/*6*/   a[l] = k;
/*7*/   smalleri--;/*removes l from smaller*/
/*8*/   q[k] = q[k] -(1.-q[l]);
/*9*/   if(q[k]<1.){
/*10*/	  greateri--;
	  smalleri++;
	  smaller[smalleri] = k;
        }
      }
      free(greater);
      free(smaller);
      return 1;
}


int generator03_03()
/*
 * Algorithm 3.3 (Alias sample), p. 48
 *
 */
{ int x;
  double u, v;

      if(a==NULL){
	printf("algo03_03: sampling invoked without setup!! run setup first\n");
	return -9999;
      }

/*2*/ u = uniform();
/*3*/ x = (int)(u*LL);
/*4*/ v = u*LL - x;
/*5*/ if(v<q[x]) 
/*6*/   return x;
/*7*/ else 
/*8*/ return a[x];
}

int free03_03()
/* frees the allocated memory */
{
      free(a);
      a = NULL;
      free(q);
      q = NULL;
      return -1;
}




