/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Code for Algorithm 5.3 (Ahrens, equal area), p. 122  
 *
 * this file: algo05_03.c
 *
 * For a demo how to use this algorithm see the file "demo_algo05_03.c".
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "uniform.h"

 static double *b=NULL, gam, (*pdfh)(double);
 static int N;

int setup05_03( double (*pdf)(double), double Af,
                int NT, double bl, double br)
/*
 * Algorithm 5.3 (Ahrens, equal area), p. 122  
 *
 * pdf... function pointer to the density
 * Af ... approximate are below the pdf
 * NT  ... approximate number of construction points
 * bl ... left border of the domain
 * br ... right border of the domain
 *
 */
{      int i;  
       if(b!=NULL){
	 free(b);
       }
       pdfh =pdf;
       b = malloc((NT*2)*sizeof(double));  



/*1*/  gam = Af/NT +2.* log(NT)/(NT*NT);
/*2*/  b[0] = bl;
       i=0;
/*3*/  do{
/*4*/    i++;
	 if( N >= 2*NT ){
	   printf("ERROR: N is too large for the arrays, probably Af wrong!\n");
	   exit(1);
	 }
	 b[i]=b[i-1]+gam/(*pdf)(b[i-1]);
/*5*/  }while(b[i]<br && (*pdf)(b[i])>0.);
/*6*/  N = i;
       printf("SETUP:\n   exact number of intervals N=%d for N-tilde=%d\n",N,NT);
       for(i=0;i<=N;i++)
	 printf("   b[%d]=%f\n",i,b[i]);
       printf("   total area below hat: %f\n\n",gam*N);
       return 1;  
}


double generator05_03()
/* is called after the setup to generator reandom variates */
{
double v,y,x,q,fjm1,fj;
 int j;  

      if(b==NULL){
	printf("ERROR generator05_03: you have to run the setup first!!\n");
	return -9999.;
      }
/*8*/ while(1){
/*9*/   v = uniform();
/*10*/  j = 1+(int)(v*N);
/*11*/  v = 1+ v*N-j;
/*12*/  if(j < N){
/*13*/    q=v*(b[j+1]-b[j]);
/*14*/    if(q<= b[j]-b[j-1]){
/*15*/      x = b[j-1] + q;
/*16*/      return x;
          }
/*17*/    else{
/*18*/      q = (q-(b[j]-b[j-1]))*(b[j]-b[j-1])/(b[j+1]-2.*b[j]+b[j-1]);
/*19*/      x = b[j-1] + q;
/*20*/      fjm1 = gam/(b[j]-b[j-1]);
            fj = gam/(b[j+1]-b[j]);
/*21*/      y = fj+(fjm1 - fj)*uniform();
/*22*/      if(y <= (*pdfh)(x))
/*23*/	      return x;
          }
        }
/*24*/  else{
/*25*/    x = b[j-1] + v*(b[j]-b[j-1]);
/*26*/    y = gam/(b[j]-b[j-1])*uniform();
/*27*/    if(y <= (*pdfh)(x)) 
/*28*/      return x;
   }
 }
}


int free05_03()
/* frees the allocated memory */
{
      free(b);
      b = NULL;
      return 1;
}


