/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Code for Algorithm 6.1 (Monotone-01, simple rej. for monotone pdf on (0,1), p. 128) 
 *
 * this file: algo06_01.c
 *
 * For a demo how to use this algorithm see the file "demo_algo06_01.c".
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "uniform.h"

static double f0 = 0., ah=0., af=0., (*pdfh)(double);


int setup06_01( double (*pdf)(double), double Afb)
/*
 * Algorithm 6.1 (Monotone-01, simple rej. for monotone pdf on (0,1), p. 128)
 *
 * pdf ... function pointer to the density, which must be monotne on (0,1)
 * Afb ... upper bound for the area below the (quasi-)density 
 *
 */
{    
      pdfh = pdf;
      af = Afb;
/*1*/ f0 = (*pdf)(0.);
/*2*/ ah = Afb*(1.+log(f0/Afb)); 
      printf("Setup:\n  f0 %f total area below hat: %f\n\n",f0,ah);
      return 1;
} 

double generator06_01()
/* is called after the setup to generate random variates */
{ double u,v,x,h;

/*3*/ while(1){
/*4*/   u = ah*uniform();
        v = uniform();
/*5*/   if(u<= af){
/*6*/     x = u/f0;
          h = f0;
        }
/*7*/   else{
/*8*/     x = (af/f0)*exp(u/af-1.);
          h = af/x;
        }
/*9*/   if(v*h<=(*pdfh)(x)) 
/*10*/    return x;
      }
}
