/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Code for Algorithm 6.5 (Simple Ratio-of-Uniforms mirror principle, p. 139)
 *
 * this file: algo06_05.c
 *
 * For a demo how to use this algorithm see the file "demo_algo06_05.c".
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "uniform.h"

static double um, vm, m, (*pdfh)(double);

int setup06_05( double (*pdf)(double), double Af, double mi)
/*
 * Algorithm 6.5 (Simple Ratio-of-Uniforms mirror principle, p. 139)
 *
 * pdf ... function pointer to the quasi density
 * Af ... area below the quasi density
 * mi ... mode of the density
 */
{     
      pdfh = pdf;
      m = mi;
/*1*/ vm = sqrt((*pdf)(m));
      um = Af/vm;
      printf("Setup06_05:\n um %f vm %f\n\n",um,vm);
      return 1;
} 

double generator06_05()
/* is called after the setup to generate random variates */
{ double u,v,x;

/*2*/ while(1){
/*3*/   u = -um + 2.*um*uniform();
        v = sqrt(2.)*vm*uniform();
/*4*/   x = u/v;
/*5*/   if(v*v<=(*pdfh)(x+m))
/*6*/     return x+m;
/*7*/   if(v*v<=(*pdfh)(x+m)+(*pdfh)(-x+m))
/*8*/     return -x+m;
      }
}
