/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag
 * 
 * Code for Algorithm 10.5 (Discrete-Unimodal-2nd Moment), p. 229
 *
 * this file: algo10_05.c
 *
 * For a demo how to use this algorithm see the file "demo_algo10_05.c".
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "uniform.h"

 static double M, theta, s2, u0, (*pmfh)(int);
 static int m;


int setup10_05(double (*pmf)(int), int mi, double s2i)
/*
 * Algorithm 10.5 (Discrete-Unimodal-2nd Moment), p. 229
 * generates discrete variates
 *
 * pmf... function pointer to unimodal probability (mass) function for k >= 0
 * mi ... mode of the pmf
 * s2i ... upper bound for teh second moment around 0 (i.e. Var + (m-mu)^2)
 *
 */
{ 
      m = mi;
      s2= s2i;
      pmfh = pmf;
      M = (*pmf)(m);
/*1*/ theta = exp(log(3.*s2*M*M)/3.);
      u0 = (M+3.*theta)*0.5;
      return 1;
}

int generator10_05()
/* is called after the setup to generate random variates */
{
      double u, v, x, y, w;
      int ix;
/*2*/ while(1){
/*3*/	u = u0*(uniform()*2.-1.);
	v = uniform();
/*4*/	if(fabs(u)<=u0-theta*0.5){
/*5*/	  x = m + u/M;
/*6*/	  y = v*M;
	}
/*7*/	else{
/*8*/	  w = 2*(fabs(u)-u0)+theta;
/*9*/	  x = m+(0.5+sqrt(3.*s2/w))* ( u<0 ? -1. : 1. );
/*10*/	  y = v*3.*s2/exp(log(fabs(x-m)-0.5)*3.);
	}
/*11*/	ix = floor(x+0.5);
/*12*/	if(y <= (*pmfh)(ix)) 
/*13*/    return ix;
      }
} 



