/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag
 * 
 * Code for Algorithm 10.7 (Discrete-Simple-RoU 2), p. 231
 *
 * this file: algo10_07.c
 *
 * For a demo how to use this algorithm see the file "demo_algo10_07.c".
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "uniform.h"

static double vm,ul,ur,(*pmfh)(int);
 static int m;


int setup10_07(double (*pmf)(int), int mi, double S, double Fm1)
/*
 * Algorithm 10.7 (Discrete-Simple-RoU 2), p. 231
 * generates discrete variates
 *
 * pmf... function pointer to unimodal probability (mass) function for k >= 0
 *        it need not sum up to 1.
 * mi ... mode of the pmf
 * S ... sum of the probabilities of the pmf
 * Fm1 ... CDF value at m-1; if unknown -1. must be used
 *
 */
{ 
      double pm;

      m = mi;
      pmfh = pmf;
/*1*/ pm = (*pmf)(m);
      vm = sqrt(pm);
/*2*/ if(Fm1>=0.){
/*3*/   ul = -Fm1*S/vm;
	ur = (1. - Fm1)*S/vm;
      }
/*4*/ else{
/*5*/   ul = -S/vm;
	ur = S/vm;
      }
     return 1;
}


int generator10_07()
/* is called after the setup to generate random variates */
{
      double u, v;
      int ix;
/*6*/ while(1){
/*7*/	u = ul+(ur-ul)*uniform();
        v = vm*uniform();
/*8*/	ix = floor(u/v)+m;
/*9*/	if(v*v <= (*pmfh)(ix)) 
/*10*/    return ix;
      }
}



