/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Code for Algorithm 12.8 (NORTU-Bivariate), p. 334 
 *
 * this file: algo12_08.c
 *
 * For a demo how to use this algorithm see the file "demo_algo12_08.c".
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "uniform.h"
#include "algo.h"
#include "statutil.h"

#ifndef M_PI
#define M_PI       3.14159265358979323846264338328      /* pi */
#endif

void generator12_08(double rho, double y[2])
/*
 * Algorithm 12.8 (NORTU-Bivariate), p. 334 
 *
 * rho .... Correlation 
 * y[2] ... to hold the result
 *
 */
{     double rhoy; 

/*1*/ rhoy = 2.*sin(M_PI*rho/6.);
/*2*/ y[0] = generator02_11();
      y[1] = generator02_11();/*normal variate with box-muller method*/
/*3*/ y[1] = rhoy * y[0]+sqrt(1.-rhoy*rhoy)*y[1];
/*4*/ y[0] = cdf_normal(y[0]);
      y[1] = cdf_normal(y[1]);
}
