/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Demo how to use Algorithm 3.2 (Indexed-Search), p. 46 
 *
 * this file: demo_algo03_02.c
 *
 * to compile this file you can use the command:
 *
 * cc -o algo03_02 demo_algo03_02.c algo03_02.c uniform.c -lm
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "algo.h"
#include "uniform.h"


/* PMF of the distribution
   you can insert your own distribution here
*/
#define PGEOM 0.5
double my_pmf(int k)
/* geomectric distribution with parameter PGEOM  */
{ return (k>0 ? PGEOM*exp((k-1.)*log(1.-PGEOM)) : 0.);
}




#define CLAS 15 /*set numberq of integers for which frequency is counted*/
#define N 100000 /* sample size */

static long int obs[CLAS];

int main()
/* demonstrates the use of the algorithm
   and compares probabilities to observed relative frequencies */
{ int i,k;
  double p;

 setseed(133248); /*set seed for uniform generator*/

 setup03_02(my_pmf, 200, 200);/* calls the setup*/

 printf("Demo for Algorithm 3.2\nfirst we display ten numbers generated from my_pmf()\n");
 for(i=0;i<10;i++){
   k =  generator03_02();/* calls the sampling*/ 
   /* it is not necessary to specify the inputs again */
   printf("k %d\n",k);
 }
 
 printf("\nNow we count the frequencies with sample size %d\n\n", N);
 for(i=0;i<CLAS;i++) obs[i]=0;

 for(i=0;i<N;i++){
   k =  generator03_02();/* calls the sampling*/ 
   if(k<CLAS) obs[k]++;
 }


 printf("  k    Probability(k)   Observed relative frequency    ratio\n");
 for( i=0; i<CLAS; i++){
   p = my_pmf(i);
   if(obs[i]>0)
     printf(" %2d      %f              %f              %f\n",
	     i,       p, obs[i]/(double)N, p/(obs[i]/(double)N));
   else 
     printf(" %2d      %f              %f                  -- \n",
	     i,       p, obs[i]/(double)N);
}

 printf("\n");

 free03_02();/* to free the allocated memory*/ 

 return 0;
}
