/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Demo how to use Algorithm 4.5 (TDR-T-convex), p. 99  
 *
 * this file: demo_algo04_05.c
 *
 * to compile this file you can use the command:
 *
 * cc -o algo04_05 demo_algo04_05.c algo04_05.c uniform.c statutil.c -lm
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "algo.h"
#include "statutil.h"
#include "uniform.h"


/* PDF of the distribution and its derivative (dpdf)
   you can insert your own distribution here
*/

double my_pdf(double x)
{return 2.*x;}
/* triangular on (0,1) */

double my_dpdf(double x)
/* derivative of the triangular pdf on (0,1) */
{ 
  if( x >= 0. && x <= 1.)
    return 2.;
  else return 0.;
}




#define CLAS 500 /*sets number of classes for the chi-square test*/

static long int obs[CLAS];

int main()
/* demonstrates the use of the algorithm
   and conducts a chi-square test */
{ int i;
 double x,cp[5];

 setseed(13324821); /*set seed for uniform generator*/

 printf("Demo for Algorithm 4.5\nfirst we display ten numbers generated from my_pdf()\n");
 cp[0] = 0.; /* left border of domain */
 cp[1] = 0.3;
 cp[2] = 0.5;
 cp[3] = 0.7;
 cp[4] = 1.; /* right border of domain */
 setup04_05(my_pdf,my_dpdf,5,cp);/* calls set-up*/
 for(i=0;i<10;i++){
   x = generator04_05();
      /* it is not necessary to specify the inputs again */
   printf("x %f\n",x);
 }
 free04_05();/* to clear the memory */

 printf("\nNow we make a chi-square test with sample size %d \n\n", CLAS*CLAS);
 for(i=0;i<CLAS;i++) obs[i]=0;
 cp[0] = 1.; /* left border of domain */
 cp[1] = 1.5;
 cp[2] = 2.;
 cp[3] = 2.5;
 cp[4] = 3.; /* right border of domain */
 setup04_05(pdf_oox1_3,dpdf_oox1_3,5,cp);/* calls set-up*/
 for(i=0;i<CLAS*CLAS;i++){
   x = generator04_05();
   obs[(int)(cdf_oox1_3(x)*CLAS)]++;
 }
 free04_05();/* to clear the memory */

 chi2test(obs,CLAS,CLAS*CLAS,1); 
 printf("\n");
 return 0;
}

