/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Demo how to use Algorithm 12.9 (Chessboard-Bivariate), p. 335 
 *
 * this file: demo_algo12_08.c
 *
 * to compile this file you can use the command:
 *
 * cc -o algo12_09 demo_algo12_09.c algo12_09.c uniform.c statutil.c -lm
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "algo.h"
#include "statutil.h"
#include "uniform.h"

#define RHO 0.8



#define CLAS 300
#define REP (CLAS*CLAS) /* number of repetitons */


static long int obs[2][CLAS];

int main()
{ int i,j;
 double x[2],rho=RHO;
 MEANCOR *mca;
    
 setseed(124885); /*set seed for uniform generator*/


  printf("\nDemo for Algorithm 12.9\nfirst we display ten generated pairs\n");
  setup12_09(rho);
  for(i=0;i<10;i++){
    generator12_09(x);
    printf("   (%f|%f) \n",x[0],x[1]);
  }


 mca = init_meancor();
 for(j=0;j<2;j++)
 for(i=0;i<CLAS;i++) obs[j][i]=0;

 printf("now we check the correlation and the uniform marginal distribution\n");
 printf("for a sample of %d pairs:\n",REP);

 for (i=0; i<REP; i++){ 
   generator12_09(x);
   for(j=0;j<2;j++) obs[j][(int)(x[j]*CLAS)]++;
   update_meancor(mca,x[0],x[1]);
 }      

 printf("desired correlation %f : observed: ",rho);
 printresult_meancor(mca);
 free_meancor(mca);
 printf("\n");
 for(j=0;j<2;j++)
   chi2test(obs[j],CLAS,CLAS*CLAS,1); 



 return 0;
}
